/*
 * Copyright (c) 2019 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef BUFFER_H
#define BUFFER_H

#include <stdlib.h> /* size_t */
#include <sys/types.h> /* ssize_t */

typedef ssize_t (*buffer_func_t)(void *user_data, void *buf, size_t bufsiz, size_t count);

struct buffer {
	char *buf;
	buffer_func_t callback;
	void *user_data;
	size_t allocated;
	size_t length;
};

void buffer_init (struct buffer *sb, void *buf, size_t bufsiz, buffer_func_t callback, void *user_data);

int buffer_flush (struct buffer *sb);
ssize_t buffer_write (struct buffer *sb, const void *src, size_t srcsiz);
int buffer_write_all (struct buffer *sb, const void *src, size_t srcsiz);

#endif
